@@ -36,7 +36,6 @@
 #include "nautilus-icon-factory-private.h"
 #include "nautilus-lib-self-check-functions.h"
 #include "nautilus-link.h"
-#include "nautilus-theme.h"
 #include "nautilus-thumbnails.h"
 #include "nautilus-trash-monitor.h"
 #include <eel/eel-debug.h>
@@ -70,6 +69,7 @@
 #define ICON_NAME_THUMBNAIL_LOADING     "gnome-fs-loading-icon"
 #define ICON_NAME_TRASH_EMPTY		"gnome-fs-trash-empty"
 #define ICON_NAME_TRASH_FULL		"gnome-fs-trash-full"
+#define ICON_NAME_HOME                  "gnome-fs-home"
 
 #define NAUTILUS_EMBLEM_NAME_PREFIX "emblem-"
 
@@ -99,6 +99,7 @@ typedef struct {
 	char *name; /* Icon name or absolute filename */
 	char *modifier;
 	guint nominal_size;
+	gboolean force_nominal;
 } CacheKey;
 
 /* The value in the same table. */
@@ -186,7 +187,8 @@ static CacheIcon *cache_icon_new                         (GdkPixbuf
 							  double                    scale_y);
 static CacheIcon *get_icon_from_cache                    (const char               *icon,
 							  const char               *modifier,
-							  guint                     nominal_size);
+							  guint                     nominal_size,
+							  gboolean		    force_nominal);
 static void nautilus_icon_factory_clear                  (void);
 
 GNOME_CLASS_BOILERPLATE (NautilusIconFactory,
@@ -318,7 +320,7 @@ load_thumbnail_frame (NautilusIconFactory *factory)
 {
 	char *image_path;
 	
-	image_path = nautilus_theme_get_image_path ("thumbnail_frame.png");
+	image_path = nautilus_pixmap_file ("thumbnail_frame.png");
 	if (factory->thumbnail_frame != NULL) {
 		g_object_unref (factory->thumbnail_frame);
 	}
@@ -372,6 +374,8 @@ nautilus_icon_factory_instance_init (NautilusIconFactory *factory)
 	
 	factory->fallback_icon = cache_icon_new (pixbuf, NULL, 1.0, 1.0);
 
+	g_object_unref(pixbuf);
+
 	factory->image_mime_types = g_hash_table_new (g_str_hash, g_str_equal);
 	for (i = 0; i < G_N_ELEMENTS (types); i++) {
 		g_hash_table_insert (factory->image_mime_types,
@@ -503,6 +507,8 @@ cache_icon_unref (CacheIcon *icon)
 	check_recently_used_list ();
 	
 	g_object_unref (icon->pixbuf);
+	
+	g_free (icon->display_name);
 	g_free (icon->embedded_text_rect);
 	g_free (icon->attach_points);
 
@@ -822,14 +828,27 @@ nautilus_icon_factory_get_icon_for_file (NautilusFile *file, gboolean embedd_tex
  	g_free (custom_uri);
 
 	file_uri = nautilus_file_get_uri (file);
+
+	if (strcmp (file_uri, "burn:///") == 0) {
+		g_free (file_uri);
+		return  g_strdup ("gnome-dev-cdrom");
+	}
+	if (strcmp (file_uri, "computer:///") == 0) {
+		g_free (file_uri);
+		return  g_strdup ("gnome-fs-client");
+	}
 	
 	if (strcmp (file_uri, EEL_TRASH_URI) == 0) {
 		g_free (file_uri);
-
 		return  g_strdup (nautilus_trash_monitor_is_empty ()
 				  ? ICON_NAME_TRASH_EMPTY : ICON_NAME_TRASH_FULL);
 	}
-	
+
+	if (nautilus_file_is_home (file)) {
+		g_free (file_uri);
+		return g_strdup (ICON_NAME_HOME);
+	}
+		
 	mime_type = nautilus_file_get_mime_type (file);
 	
 	file_info = nautilus_file_peek_vfs_file_info (file);
@@ -957,8 +976,8 @@ nautilus_icon_factory_get_emblem_icons_for_file (NautilusFile *file,
 	return g_list_reverse (icons);
 }
 
-static guint
-get_larger_icon_size (guint size)
+guint
+nautilus_icon_factory_get_larger_icon_size (guint size)
 {
 	if (size < NAUTILUS_ICON_SIZE_SMALLEST) {
 		return NAUTILUS_ICON_SIZE_SMALLEST;
@@ -984,8 +1003,8 @@ get_larger_icon_size (guint size)
 	return NAUTILUS_ICON_SIZE_LARGEST;
 }
 
-static guint
-get_smaller_icon_size (guint size)
+guint
+nautilus_icon_factory_get_smaller_icon_size (guint size)
 {
 	if (size > NAUTILUS_ICON_SIZE_LARGEST) {
 		return NAUTILUS_ICON_SIZE_LARGEST;
@@ -1084,6 +1103,7 @@ static GdkPixbuf *
 load_icon_file (const char    *filename,
 		guint          base_size,
 		guint          nominal_size,
+		gboolean       force_nominal,
 		double        *scale_x,
 		double        *scale_y)
 {
@@ -1091,6 +1111,7 @@ load_icon_file (const char    *filename,
 	int width, height, size;
 	double scale;
 	gboolean is_thumbnail;
+        gboolean add_frame = FALSE;
 
 	*scale_x = 1.0;
 	*scale_y = 1.0;
@@ -1098,7 +1119,7 @@ load_icon_file (const char    *filename,
 	if (path_represents_svg_image (filename)) {
 		pixbuf = load_pixbuf_svg (filename,
 					  nominal_size,
-					  base_size,
+					  force_nominal ? 0 : base_size,
 					  scale_x, scale_y);
 	} else {
 		is_thumbnail = strstr (filename, "/.thumbnails/")  != NULL;
@@ -1115,15 +1136,25 @@ load_icon_file (const char    *filename,
 			return NULL;
 		}
 		
-		if (base_size == 0) {
+		if (force_nominal) {
+			width = gdk_pixbuf_get_width (pixbuf); 
+			height = gdk_pixbuf_get_height (pixbuf);
+			base_size = MAX (width, height);                        
+		} else if (base_size == 0) {
 			if (is_thumbnail) {
 				base_size = 128 * NAUTILUS_ICON_SIZE_STANDARD / NAUTILUS_ICON_SIZE_THUMBNAIL;
 			} else {
 				width = gdk_pixbuf_get_width (pixbuf); 
 				height = gdk_pixbuf_get_height (pixbuf);
 				size = MAX (width, height);
-				if (size > NAUTILUS_ICON_SIZE_STANDARD + 5) {
-					base_size = size;
+                                if (size > NAUTILUS_ICON_SIZE_THUMBNAIL && !gdk_pixbuf_get_has_alpha(pixbuf)) {
+                                        add_frame=TRUE;
+                                }
+                                if (size >  nominal_size * NAUTILUS_ICON_SIZE_THUMBNAIL / NAUTILUS_ICON_SIZE_STANDARD) {
+                                        base_size = size * NAUTILUS_ICON_SIZE_STANDARD / NAUTILUS_ICON_SIZE_THUMBNAIL;
+                                }
+                                else if (size > NAUTILUS_ICON_SIZE_STANDARD) {
+					base_size = nominal_size;
 				} else {
 					/* Don't scale up small icons */
 					base_size = NAUTILUS_ICON_SIZE_STANDARD;
@@ -1141,13 +1172,17 @@ load_icon_file (const char    *filename,
 		}
 	}
 
+        if (add_frame){
+                nautilus_thumbnail_frame_image(&pixbuf);
+        }
 	return pixbuf;
 }
 
 static CacheIcon *
 create_normal_cache_icon (const char *icon,
 			  const char *modifier,
-			  guint       nominal_size)
+			  guint       nominal_size,
+			  gboolean    force_nominal)
 {
 	NautilusIconFactory *factory;
 	const char *filename;
@@ -1173,7 +1208,7 @@ create_normal_cache_icon (const char *icon,
 		 *        before the extension */
 		if (stat (icon, &statbuf) == 0 &&
 		    S_ISREG (statbuf.st_mode)) {
-			filename = g_strdup (icon);
+			filename = icon;
 			mtime = statbuf.st_mtime;
 		}
 	} else {
@@ -1208,6 +1243,7 @@ create_normal_cache_icon (const char *icon,
 	pixbuf = load_icon_file (filename,
 				 base_size,
 				 nominal_size,
+				 force_nominal,
 				 &scale_x, &scale_y);
 	if (pixbuf == NULL) {
 		if (info) {
@@ -1232,11 +1268,14 @@ create_normal_cache_icon (const char *icon,
  * If @picky is true, then only an unscaled icon is acceptable.
  * Also, if @picky is true, the icon must be a custom icon if
  * @custom is true or a standard icon is @custom is false.
+ * If @force_nominal is #TRUE, the returned icon will be guaranteed
+ * to be smaller than the nominal size
  */
 static CacheIcon *
 get_icon_from_cache (const char *icon,
 		     const char *modifier,
-		     guint       nominal_size)
+		     guint       nominal_size,
+		     gboolean    force_nominal)
 {
 	NautilusIconFactory *factory;
 	GHashTable *hash_table;
@@ -1258,6 +1297,7 @@ get_icon_from_cache (const char *icon,
 	lookup_key.name = (char *)icon;
 	lookup_key.modifier = (char *)modifier;
 	lookup_key.nominal_size = nominal_size;
+	lookup_key.force_nominal = force_nominal;
 
 	if (g_hash_table_lookup_extended (hash_table, &lookup_key,
 					  &key_in_table, &value)) {
@@ -1288,12 +1328,14 @@ get_icon_from_cache (const char *icon,
 		
 		cached_icon = create_normal_cache_icon (icon,
 							modifier,
-							nominal_size);
+							nominal_size,
+							force_nominal);
 		/* Try to fallback without modifier */
 		if (cached_icon == NULL && modifier != NULL) {
 			cached_icon = create_normal_cache_icon (icon,
 								NULL,
-								nominal_size);
+								nominal_size,
+								force_nominal);
 		}
 		
 		if (cached_icon == NULL) {
@@ -1306,6 +1348,7 @@ get_icon_from_cache (const char *icon,
 		key->name = g_strdup (icon);
 		key->modifier = g_strdup (modifier);
 		key->nominal_size = nominal_size;
+		key->force_nominal = force_nominal;
 
 		g_hash_table_insert (hash_table, key, cached_icon);
 	}
@@ -1322,14 +1365,15 @@ get_icon_from_cache (const char *icon,
         return cached_icon;
 }
 
-GdkPixbuf *
-nautilus_icon_factory_get_pixbuf_for_icon (const char                  *icon,
-					   const char                  *modifier,
-					   guint                        nominal_size,
-					   NautilusEmblemAttachPoints  *attach_points,
-					   GdkRectangle                *embedded_text_rect,
-					   gboolean                     wants_default,
-					   char                       **display_name)
+static GdkPixbuf *
+nautilus_icon_factory_get_pixbuf_for_icon_internal (const char                  *icon,
+						    const char                  *modifier,
+						    guint                        nominal_size,
+						    gboolean                     force_size,
+						    NautilusEmblemAttachPoints  *attach_points,
+						    GdkRectangle                *embedded_text_rect,
+						    gboolean                     wants_default,
+						    char                       **display_name)
 {
 	NautilusIconFactory *factory;
 	CacheIcon *cached_icon;
@@ -1339,7 +1383,8 @@ nautilus_icon_factory_get_pixbuf_for_icon (const char                  *icon,
 	factory = get_icon_factory ();
 	cached_icon = get_icon_from_cache (icon,
 					   modifier,
-					   nominal_size);
+					   nominal_size,
+					   force_size);
 
 	if (attach_points != NULL) {
 		if (cached_icon->attach_points != NULL) {
@@ -1381,6 +1426,35 @@ nautilus_icon_factory_get_pixbuf_for_icon (const char                  *icon,
 	return pixbuf;
 }
 
+GdkPixbuf *
+nautilus_icon_factory_get_pixbuf_for_icon (const char                  *icon,
+					   const char                  *modifier,
+					   guint                        nominal_size,
+					   NautilusEmblemAttachPoints  *attach_points,
+					   GdkRectangle                *embedded_text_rect,
+					   gboolean                     wants_default,
+					   char                       **display_name)
+{
+	return nautilus_icon_factory_get_pixbuf_for_icon_internal (icon, modifier, nominal_size, FALSE,
+								   attach_points, embedded_text_rect,
+								   wants_default, display_name);
+}
+
+GdkPixbuf *
+nautilus_icon_factory_get_pixbuf_for_icon_force_size (const char                  *icon,
+						      const char                  *modifier,
+						      guint                        nominal_size,
+						      NautilusEmblemAttachPoints  *attach_points,
+						      GdkRectangle                *embedded_text_rect,
+						      gboolean                     wants_default,
+						      char                       **display_name)
+{
+	return nautilus_icon_factory_get_pixbuf_for_icon_internal (icon, modifier, nominal_size, TRUE,
+								   attach_points, embedded_text_rect,
+								   wants_default, display_name);
+}
+
+
 static guint
 cache_key_hash (gconstpointer p)
 {
@@ -1390,12 +1464,12 @@ cache_key_hash (gconstpointer p)
 	key = p;
 
 	hash =  g_str_hash (key->name) ^
-		(key->nominal_size << 4);
+		((key->nominal_size << 4) + (gint)key->force_nominal);
 	
 	if (key->modifier) {
 		hash ^= g_str_hash (key->modifier);
 	}
-		
+
 	return hash;
 }
 
@@ -1409,6 +1483,7 @@ cache_key_equal (gconstpointer a, gconstpointer b)
 
 	return eel_strcmp (key_a->name, key_b->name) == 0 &&
 		key_a->nominal_size ==  key_b->nominal_size &&
+		key_a->force_nominal == key_b->force_nominal &&
 		eel_strcmp (key_a->modifier, key_b->modifier) == 0;
 }
 
@@ -1440,13 +1515,22 @@ nautilus_get_icon_size_for_zoom_level (NautilusZoomLevel zoom_level)
 	g_return_val_if_reached (NAUTILUS_ICON_SIZE_STANDARD);
 }
 
+float
+nautilus_get_relative_icon_size_for_zoom_level (NautilusZoomLevel zoom_level)
+{
+	return (float)nautilus_get_icon_size_for_zoom_level (zoom_level) / NAUTILUS_ICON_SIZE_STANDARD;
+}
+
+
+
 /* Convenience cover for nautilus_icon_factory_get_icon_for_file
  * and nautilus_icon_factory_get_pixbuf_for_icon.
  */
-GdkPixbuf *
-nautilus_icon_factory_get_pixbuf_for_file (NautilusFile *file,
-					   const char *modifier,
-					   guint size_in_pixels)
+static GdkPixbuf *
+nautilus_icon_factory_get_pixbuf_for_file_internal (NautilusFile *file,
+						    const char *modifier,
+						    guint size_in_pixels,
+						    gboolean force_size)
 {
 	char *icon;
 	GdkPixbuf *pixbuf;
@@ -1458,27 +1542,51 @@ nautilus_icon_factory_get_pixbuf_for_file (NautilusFile *file,
 		return NULL;
 	}
 
-	pixbuf = nautilus_icon_factory_get_pixbuf_for_icon (icon, modifier,
-							    size_in_pixels,
-							    NULL, NULL,
-							    TRUE, NULL);
+	pixbuf = nautilus_icon_factory_get_pixbuf_for_icon_internal (icon, modifier,
+								     size_in_pixels,
+								     force_size,
+								     NULL, NULL,
+								     TRUE, NULL);
 	
 	g_free (icon);
 
 	return pixbuf;
 }
 
+GdkPixbuf *
+nautilus_icon_factory_get_pixbuf_for_file (NautilusFile *file,
+					   const char *modifier,
+					   guint size_in_pixels)
+{
+	return nautilus_icon_factory_get_pixbuf_for_file_internal (file,
+								   modifier,
+								   size_in_pixels,
+								   FALSE);
+}
+
+GdkPixbuf *
+nautilus_icon_factory_get_pixbuf_for_file_force_size (NautilusFile *file,
+						      const char *modifier,
+						      guint size_in_pixels)
+{
+	return nautilus_icon_factory_get_pixbuf_for_file_internal (file,
+								   modifier,
+								   size_in_pixels,
+								   TRUE);
+}
+
 /* Convenience routine for getting a pixbuf from an icon name. */
 GdkPixbuf *
 nautilus_icon_factory_get_pixbuf_from_name (const char *icon_name,
 					    const char *modifier,
 					    guint size_in_pixels,
+					    gboolean force_size,
 					    char **display_name)
 {
-	return nautilus_icon_factory_get_pixbuf_for_icon (icon_name, modifier,
-							  size_in_pixels,
-							  NULL, NULL,
-							  TRUE, display_name);
+	return nautilus_icon_factory_get_pixbuf_for_icon_internal (icon_name, modifier,
+								   size_in_pixels, force_size,
+								   NULL, NULL,
+								   TRUE, display_name);
 }
 									  
 GdkPixbuf *
@@ -1520,45 +1628,45 @@ nautilus_self_check_icon_factory (void)
 	EEL_CHECK_INTEGER_RESULT (nautilus_get_icon_size_for_zoom_level (5), 96);
 	EEL_CHECK_INTEGER_RESULT (nautilus_get_icon_size_for_zoom_level (6), 192);
 
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (0), 12);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (1), 12);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (11), 12);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (12), 20);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (19), 20);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (20), 24);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (23), 24);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (24), 36);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (35), 36);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (36), 48);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (47), 48);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (48), 72);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (71), 72);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (72), 96);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (95), 96);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (96), 192);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (191), 192);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (192), 192);
-	EEL_CHECK_INTEGER_RESULT (get_larger_icon_size (0xFFFFFFFF), 192);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (0), 12);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (1), 12);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (11), 12);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (12), 20);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (19), 20);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (20), 24);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (23), 24);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (24), 36);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (35), 36);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (36), 48);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (47), 48);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (48), 72);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (71), 72);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (72), 96);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (95), 96);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (96), 192);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (191), 192);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (192), 192);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_larger_icon_size (0xFFFFFFFF), 192);
 
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (0), 12);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (1), 12);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (11), 12);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (12), 12);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (20), 12);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (21), 20);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (24), 20);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (25), 24);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (36), 24);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (37), 36);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (48), 36);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (49), 48);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (72), 48);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (73), 72);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (96), 72);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (97), 96);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (192), 96);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (193), 192);
-	EEL_CHECK_INTEGER_RESULT (get_smaller_icon_size (0xFFFFFFFF), 192);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (0), 12);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (1), 12);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (11), 12);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (12), 12);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (20), 12);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (21), 20);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (24), 20);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (25), 24);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (36), 24);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (37), 36);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (48), 36);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (49), 48);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (72), 48);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (73), 72);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (96), 72);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (97), 96);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (192), 96);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (193), 192);
+	EEL_CHECK_INTEGER_RESULT (nautilus_icon_factory_get_smaller_icon_size (0xFFFFFFFF), 192);
 }
 
 #endif /* ! NAUTILUS_OMIT_SELF_CHECK */
