@@ -38,6 +38,7 @@
 #include <eel/eel-gtk-macros.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkdialog.h>
+#include <gtk/gtkbindings.h>
 #include <gtk/gtkcellrendererpixbuf.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkentry.h>
@@ -96,13 +97,17 @@ struct FMListViewDetails {
 	GtkTargetList *source_target_list;
 
 	GtkTreePath *double_click_path[2]; /* Both clicks in a double click need to be on the same row */
-	
+
+	GtkTreePath *new_selection_path;   /* Path of the new selection after removing a file */
+
 	guint drag_button;
 	int drag_x;
 	int drag_y;
 
 	gboolean drag_started;
 
+	gboolean ignore_button_release;
+
 	gboolean row_selected_on_button_down;
 
 	/* typeahead selection state */
@@ -116,6 +121,11 @@ struct FMListViewDetails {
 	char *original_name;
 };
 
+struct SelectionForeachData {
+	GList *list;
+	GtkTreeSelection *selection;
+};
+
 /*
  * The row height should be large enough to not clip emblems.
  * Computing this would be costly, so we just choose a number
@@ -123,6 +133,9 @@ struct FMListViewDetails {
  */
 #define LIST_VIEW_MINIMUM_ROW_HEIGHT	28
 
+/* We wait two seconds after row is collapsed to unload the subdirectory */
+#define COLLAPSE_TO_UNLOAD_DELAY 2000 
+
 static int                      click_policy_auto_value;
 static char *              	default_sort_order_auto_value;
 static gboolean			default_sort_reversed_auto_value;
@@ -130,15 +143,16 @@ static NautilusZoomLevel        default_zoom_level_auto_value;
 static GList *                  default_visible_columns_auto_value;
 static GList *                  default_column_order_auto_value;
 
-static GList *fm_list_view_get_selection   (FMDirectoryView   *view);
-static void   fm_list_view_set_zoom_level  (FMListView        *view,
-					    NautilusZoomLevel  new_level,
-					    gboolean           always_set_level);
-static void   fm_list_view_scale_font_size (FMListView        *view,
-					    NautilusZoomLevel  new_level);
-static void   fm_list_view_scroll_to_file  (FMListView        *view,
-					    NautilusFile      *file);
-static void   fm_list_view_iface_init      (NautilusViewIface *iface);
+static GList *fm_list_view_get_selection                   (FMDirectoryView   *view);
+static GList *fm_list_view_get_selection_for_file_transfer (FMDirectoryView   *view);
+static void   fm_list_view_set_zoom_level                  (FMListView        *view,
+							    NautilusZoomLevel  new_level,
+							    gboolean           always_set_level);
+static void   fm_list_view_scale_font_size                 (FMListView        *view,
+							    NautilusZoomLevel  new_level);
+static void   fm_list_view_scroll_to_file                  (FMListView        *view,
+							    NautilusFile      *file);
+static void   fm_list_view_iface_init                      (NautilusViewIface *iface);
 
 
 
@@ -245,8 +259,7 @@ fm_list_view_did_not_drag (FMListView *view,
 		if ((event->button == 1 || event->button == 2)
 		    && ((event->state & GDK_CONTROL_MASK) != 0 ||
 			(event->state & GDK_SHIFT_MASK) == 0)
-		    && view->details->row_selected_on_button_down
-		    && (click_policy_auto_value == NAUTILUS_CLICK_POLICY_DOUBLE)) {
+		    && view->details->row_selected_on_button_down) {
 			if (!button_event_modifies_selection (event)) {
 				gtk_tree_selection_unselect_all (selection);
 				gtk_tree_selection_select_path (selection, path);
@@ -302,35 +315,46 @@ drag_data_get_callback (GtkWidget *widget,
 }
 
 static void
-selection_foreach (GtkTreeModel *model,
-		   GtkTreePath *path,
-		   GtkTreeIter *iter,
-		   gpointer data)
+filtered_selection_foreach (GtkTreeModel *model,
+			    GtkTreePath *path,
+			    GtkTreeIter *iter,
+			    gpointer data)
 {
-	GList **list;
+	struct SelectionForeachData *selection_data;
+	GtkTreeIter parent;
+	GtkTreeIter child;
 	
-	list = (GList**)data;
-	
-	*list = g_list_prepend (*list, 
-				gtk_tree_row_reference_new (model, path));
+	selection_data = data;
+
+	/* If the parent folder is also selected, don't include this file in the
+	 * file operation, since that would copy it to the toplevel target instead
+	 * of keeping it as a child of the copied folder
+	 */
+	child = *iter;
+	while (gtk_tree_model_iter_parent (model, &parent, &child)) {
+		if (gtk_tree_selection_iter_is_selected (selection_data->selection,
+							 &parent)) {
+			return;
+		}
+		child = parent;
+	}
 	
+	selection_data->list = g_list_prepend (selection_data->list, 
+					       gtk_tree_row_reference_new (model, path));
 }
 
 static GList *
-get_selection_refs (GtkTreeView *tree_view)
+get_filtered_selection_refs (GtkTreeView *tree_view)
 {
-	GtkTreeSelection *selection;
-	GList *ref_list;
-	
-	ref_list = NULL;
-	
-	selection = gtk_tree_view_get_selection (tree_view);
-	gtk_tree_selection_selected_foreach (selection, 
-					     selection_foreach, 
-					     &ref_list);
-	ref_list = g_list_reverse (ref_list);
+	struct SelectionForeachData selection_data;
 
-	return ref_list;
+	selection_data.list = NULL;
+	selection_data.selection = gtk_tree_view_get_selection (tree_view);
+	
+	gtk_tree_selection_selected_foreach (selection_data.selection, 
+					     filtered_selection_foreach, 
+					     &selection_data);
+	return g_list_reverse (selection_data.list);
 }
 
 static void
@@ -411,7 +435,7 @@ motion_notify_callback (GtkWidget *widget,
 			stop_drag_check (view);
 			view->details->drag_started = TRUE;
 			
-			ref_list = get_selection_refs (GTK_TREE_VIEW (widget));
+			ref_list = get_filtered_selection_refs (GTK_TREE_VIEW (widget));
 			g_object_set_data_full (G_OBJECT (context),
 						"drag-info",
 						ref_list,
@@ -427,8 +451,10 @@ motion_notify_callback (GtkWidget *widget,
 				gtk_drag_set_icon_default (context);
 			}
 		}		      
+		return TRUE;
 	}
-	return TRUE;
+	
+	return FALSE;
 }
 
 static void
@@ -455,6 +481,8 @@ button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callba
 	static gint64 last_click_time = 0;
 	static int click_count = 0;
 	int double_click_time;
+	int expander_size, horizontal_separator;
+	gboolean on_expander;
 
 	view = FM_LIST_VIEW (callback_data);
 	tree_view = GTK_TREE_VIEW (widget);
@@ -490,7 +518,8 @@ button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callba
 		return TRUE;
 	}
 
-	
+	view->details->ignore_button_release = FALSE;
+
 	call_parent = TRUE;
 	allow_drag = FALSE;
 	if (gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y,
@@ -543,12 +572,23 @@ button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callba
 			if ((event->button == 1 || event->button == 2) &&
 			    ((event->state & GDK_CONTROL_MASK) != 0 ||
 			     (event->state & GDK_SHIFT_MASK) == 0)) {
+				gtk_widget_style_get (widget,
+						      "expander-size", &expander_size,
+						      "horizontal-separator", &horizontal_separator,
+						      NULL);
+				on_expander = (event->x <= horizontal_separator / 2 +
+					       gtk_tree_path_get_depth (path) * expander_size);
+
+				
 				view->details->row_selected_on_button_down = gtk_tree_selection_path_is_selected (selection, path);
 				if (view->details->row_selected_on_button_down) {
-					call_parent = FALSE;
+					call_parent = on_expander;
+					view->details->ignore_button_release = call_parent;
 				} else if  ((event->state & GDK_CONTROL_MASK) != 0) {
 					call_parent = FALSE;
 					gtk_tree_selection_select_path (selection, path);
+				} else {
+					view->details->ignore_button_release = on_expander;
 				}
 			}
 		
@@ -606,9 +646,9 @@ button_release_callback (GtkWidget *widget,
 
 	if (event->button == view->details->drag_button) {
 		stop_drag_check (view);
-		if (!view->details->drag_started) {
+		if (!view->details->drag_started &&
+		    !view->details->ignore_button_release) {
 			fm_list_view_did_not_drag (view, event);
-			return TRUE;
 		}
 	}
 	return FALSE;
@@ -736,6 +776,115 @@ popup_menu_callback (GtkWidget *widget, gpointer callback_data)
 	return TRUE;
 }
 
+static void
+subdirectory_done_loading_callback (NautilusDirectory *directory, FMListView *view)
+{
+	fm_list_model_subdirectory_done_loading (view->details->model, directory);
+}
+
+static void
+row_expanded_callback (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer callback_data)
+{
+ 	FMListView *view;
+ 	NautilusDirectory *directory;
+
+	view = FM_LIST_VIEW (callback_data);
+
+	if (fm_list_model_load_subdirectory (view->details->model, path, &directory)) {
+		fm_directory_view_add_subdirectory (FM_DIRECTORY_VIEW (view), directory);
+		
+		if (nautilus_directory_are_all_files_seen (directory)) {
+			fm_list_model_subdirectory_done_loading (view->details->model,
+								 directory);
+		} else {
+			g_signal_connect_object (directory, "done_loading",
+						 G_CALLBACK (subdirectory_done_loading_callback),
+						 view, 0);
+		}
+		
+		nautilus_directory_unref (directory);
+	}
+}
+
+struct UnloadDelayData {
+	NautilusFile *file;
+	FMListView *view;
+};
+
+static gboolean
+unload_file_timeout (gpointer data)
+{
+	struct UnloadDelayData *unload_data = data;
+	GtkTreeIter iter;
+	FMListModel *model;
+	GtkTreePath *path;
+
+	if (unload_data->view != NULL) {
+		model = unload_data->view->details->model;
+		if (fm_list_model_get_tree_iter_from_file (model,
+							   unload_data->file,
+							   &iter)) {
+			path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
+			if (!gtk_tree_view_row_expanded (unload_data->view->details->tree_view,
+							 path)) {
+				fm_list_model_unload_subdirectory (model, &iter);
+			}
+			gtk_tree_path_free (path);
+		}
+	}
+
+	eel_remove_weak_pointer (&unload_data->view);
+	
+	
+	g_object_unref (unload_data->file);
+	g_free (unload_data);
+	return FALSE;
+}
+
+static void
+row_collapsed_callback (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer callback_data)
+{
+ 	FMListView *view;
+ 	NautilusFile *file;
+	struct UnloadDelayData *unload_data;
+	
+	view = FM_LIST_VIEW (callback_data);
+		
+	gtk_tree_model_get (GTK_TREE_MODEL (view->details->model), 
+			    iter, 
+			    FM_LIST_MODEL_FILE_COLUMN, &file,
+			    -1);
+	
+	unload_data = g_new (struct UnloadDelayData, 1);
+	unload_data->view = view;
+	unload_data->file = file;
+
+	eel_add_weak_pointer (&unload_data->view);
+	
+	g_timeout_add (COLLAPSE_TO_UNLOAD_DELAY,
+		       unload_file_timeout,
+		       unload_data);
+}
+
+
+static void
+subdirectory_unloaded_callback (FMListModel *model,
+				NautilusDirectory *directory,
+				gpointer callback_data)
+{
+	FMListView *view;
+	
+	g_return_if_fail (FM_IS_LIST_MODEL (model));
+	g_return_if_fail (NAUTILUS_IS_DIRECTORY (directory));
+
+	view = FM_LIST_VIEW(callback_data);
+	
+	g_signal_handlers_disconnect_by_func (directory,
+					      G_CALLBACK (subdirectory_done_loading_callback),
+					      view);
+	fm_directory_view_remove_subdirectory (FM_DIRECTORY_VIEW (view), directory);
+}
+
 static gboolean
 key_press_callback (GtkWidget *widget, GdkEventKey *event, gpointer callback_data)
 {
@@ -824,6 +973,13 @@ sort_column_changed_callback (GtkTreeSortable *sortable,
 				    default_reversed_attr, reversed_attr);
 }
 
+static void
+cell_renderer_editing_canceled (GtkCellRendererText *cell,
+				FMListView          *view)
+{
+	fm_directory_view_unfreeze_updates (FM_DIRECTORY_VIEW (view));
+}
+
 static void
 cell_renderer_edited (GtkCellRendererText *cell,
 		      const char          *path_str,
@@ -870,6 +1026,7 @@ cell_renderer_edited (GtkCellRendererText *cell,
 		      "editable", FALSE,
 		      NULL);
 
+	fm_directory_view_unfreeze_updates (FM_DIRECTORY_VIEW (view));
 }
 
 static char *
@@ -925,6 +1082,13 @@ list_view_handle_uri_list (NautilusTreeViewDragDest *dest, const char *item_uris
 						item_uris, action, x, y);
 }
 
+static void
+list_view_handle_text (NautilusTreeViewDragDest *dest, const char *text,
+		       GdkDragAction action, int x, int y, FMListView *view)
+{
+	fm_directory_view_handle_text_drop (FM_DIRECTORY_VIEW (view),
+					    text, action, x, y);
+}
 
 static void
 move_copy_items_callback (NautilusTreeViewDragDest *dest,
@@ -1003,7 +1167,8 @@ create_and_set_up_tree_view (FMListView *view)
 {
 	GtkCellRenderer *cell;
 	GtkTreeViewColumn *column;
-	GtkTargetEntry *drag_types;
+	const GtkTargetEntry *drag_types;
+	GtkBindingSet *binding_set;
 	AtkObject *atk_obj;
 	int num_drag_types;	
 	GList *nautilus_columns;
@@ -1015,6 +1180,10 @@ create_and_set_up_tree_view (FMListView *view)
 							(GDestroyNotify)g_free,
 							NULL);
 
+	/* Don't handle backspace key. It's used to open the parent folder. */
+	binding_set = gtk_binding_set_by_class (GTK_WIDGET_GET_CLASS (view->details->tree_view));
+	gtk_binding_entry_clear (binding_set, GDK_BackSpace, 0);
+
 	fm_list_model_get_drag_types (&drag_types, &num_drag_types);
 	
 	view->details->drag_dest = 
@@ -1036,6 +1205,8 @@ create_and_set_up_tree_view (FMListView *view)
 				 G_CALLBACK (list_view_handle_url), view, 0);
 	g_signal_connect_object (view->details->drag_dest, "handle_uri_list",
 				 G_CALLBACK (list_view_handle_uri_list), view, 0);
+	g_signal_connect_object (view->details->drag_dest, "handle_text",
+				 G_CALLBACK (list_view_handle_text), view, 0);
 
 	g_signal_connect_object (gtk_tree_view_get_selection (view->details->tree_view),
 				 "changed",
@@ -1053,6 +1224,10 @@ create_and_set_up_tree_view (FMListView *view)
 				 G_CALLBACK (key_press_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "popup_menu",
                                  G_CALLBACK (popup_menu_callback), view, 0);
+	g_signal_connect_object (view->details->tree_view, "row_expanded",
+                                 G_CALLBACK (row_expanded_callback), view, 0);
+	g_signal_connect_object (view->details->tree_view, "row_collapsed",
+                                 G_CALLBACK (row_collapsed_callback), view, 0);
 	
 	view->details->model = g_object_new (FM_TYPE_LIST_MODEL, NULL);
 	gtk_tree_view_set_model (view->details->tree_view, GTK_TREE_MODEL (view->details->model));
@@ -1062,6 +1237,9 @@ create_and_set_up_tree_view (FMListView *view)
 
 	g_signal_connect_object (view->details->model, "sort_column_changed",
 				 G_CALLBACK (sort_column_changed_callback), view, 0);
+	
+	g_signal_connect_object (view->details->model, "subdirectory_unloaded",
+				 G_CALLBACK (subdirectory_unloaded_callback), view, 0);
 
 	view->details->source_target_list = 
 		gtk_target_list_new (drag_types, num_drag_types);
@@ -1117,6 +1295,7 @@ create_and_set_up_tree_view (FMListView *view)
 			cell = gtk_cell_renderer_text_new ();
 			view->details->file_name_cell = (GtkCellRendererText *)cell;
 			g_signal_connect (cell, "edited", G_CALLBACK (cell_renderer_edited), view);
+			g_signal_connect (cell, "editing-canceled", G_CALLBACK (cell_renderer_editing_canceled), view);
 			
 			gtk_tree_view_column_pack_start (view->details->file_name_column, cell, TRUE);
 			gtk_tree_view_column_set_attributes (view->details->file_name_column, cell,
@@ -1163,7 +1342,10 @@ create_and_set_up_tree_view (FMListView *view)
 static void
 fm_list_view_add_file (FMDirectoryView *view, NautilusFile *file)
 {
-	fm_list_model_add_file (FM_LIST_VIEW (view)->details->model, file);
+	FMListModel *model;
+	
+	model = FM_LIST_VIEW (view)->details->model;
+	fm_list_model_add_file (model, file);
 }
 
 static GList *
@@ -1357,8 +1539,10 @@ fm_list_view_get_selection_foreach_func (GtkTreeModel *model, GtkTreePath *path,
 			    FM_LIST_MODEL_FILE_COLUMN, &file,
 			    -1);
 
-	nautilus_file_ref (file);
-	(* list) = g_list_prepend ((* list), file);
+	if (file != NULL) {
+		nautilus_file_ref (file);
+		(* list) = g_list_prepend ((* list), file);
+	}
 }
 
 static GList *
@@ -1371,9 +1555,58 @@ fm_list_view_get_selection (FMDirectoryView *view)
 	gtk_tree_selection_selected_foreach (gtk_tree_view_get_selection (FM_LIST_VIEW (view)->details->tree_view),
 					     fm_list_view_get_selection_foreach_func, &list);
 
-	return list;
+	return g_list_reverse (list);
 }
 
+static void
+fm_list_view_get_selection_for_file_transfer_foreach_func (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
+{
+	NautilusFile *file;
+	struct SelectionForeachData *selection_data;
+	GtkTreeIter parent, child;
+
+	selection_data = data;
+
+	gtk_tree_model_get (model, iter,
+			    FM_LIST_MODEL_FILE_COLUMN, &file,
+			    -1);
+
+	if (file != NULL) {
+		/* If the parent folder is also selected, don't include this file in the
+		 * file operation, since that would copy it to the toplevel target instead
+		 * of keeping it as a child of the copied folder
+		 */
+		child = *iter;
+		while (gtk_tree_model_iter_parent (model, &parent, &child)) {
+			if (gtk_tree_selection_iter_is_selected (selection_data->selection,
+								 &parent)) {
+				return;
+			}
+			child = parent;
+		}
+		
+		nautilus_file_ref (file);
+		selection_data->list = g_list_prepend (selection_data->list, file);
+	}
+}
+
+
+static GList *
+fm_list_view_get_selection_for_file_transfer (FMDirectoryView *view)
+{
+	struct SelectionForeachData selection_data;
+
+	selection_data.list = NULL;
+	selection_data.selection = gtk_tree_view_get_selection (FM_LIST_VIEW (view)->details->tree_view);
+
+	gtk_tree_selection_selected_foreach (selection_data.selection,
+					     fm_list_view_get_selection_for_file_transfer_foreach_func, &selection_data);
+
+	return g_list_reverse (selection_data.list);
+}
+
+
+
 
 static guint
 fm_list_view_get_item_count (FMDirectoryView *view)
@@ -1389,15 +1622,33 @@ fm_list_view_is_empty (FMDirectoryView *view)
 	return fm_list_model_is_empty (FM_LIST_VIEW (view)->details->model);
 }
 
+static void
+fm_list_view_end_file_changes (FMDirectoryView *view)
+{
+	FMListView *list_view;
+
+	list_view = FM_LIST_VIEW (view);
+
+	if (list_view->details->new_selection_path) {
+		gtk_tree_view_set_cursor (list_view->details->tree_view,
+					  list_view->details->new_selection_path,
+					  NULL, FALSE);
+		gtk_tree_path_free (list_view->details->new_selection_path);
+		list_view->details->new_selection_path = NULL;
+	}
+}
+
 static void
 fm_list_view_remove_file (FMDirectoryView *view, NautilusFile *file)
 {
 	GtkTreePath *path;
+	GtkTreePath *file_path;
 	GtkTreeIter iter;
 	GtkTreeIter temp_iter;
 	GtkTreeRowReference* row_reference;
 	FMListView *list_view;
 	GtkTreeModel* tree_model; 
+	GtkTreeSelection *selection;
 	
 	path = NULL;
 	row_reference = NULL;
@@ -1405,31 +1656,37 @@ fm_list_view_remove_file (FMDirectoryView *view, NautilusFile *file)
 	tree_model = GTK_TREE_MODEL(list_view->details->model);
 	
 	if(fm_list_model_get_tree_iter_from_file (list_view->details->model, file, &iter)) {
-	   temp_iter = iter;
-	  
-	   /* get reference for next element in the list view. If the element to be deleted is the 
-	    * last one, get reference to previous element. If there is only one element in view
-            * no need to select anything. */		
-	   
-	   if(gtk_tree_model_iter_next (tree_model, &iter)) {
-              path = gtk_tree_model_get_path (tree_model, &iter);
-              row_reference = gtk_tree_row_reference_new (tree_model, path);
-           } else {
-              path = gtk_tree_model_get_path (tree_model, &temp_iter);
-              if(gtk_tree_path_prev (path)) {
-                 row_reference = gtk_tree_row_reference_new (tree_model, path);
+	   selection = gtk_tree_view_get_selection (list_view->details->tree_view);
+	   file_path = gtk_tree_model_get_path (tree_model, &iter);
+
+	   if (gtk_tree_selection_path_is_selected (selection, file_path)) {
+	       /* get reference for next element in the list view. If the element to be deleted is the 
+	        * last one, get reference to previous element. If there is only one element in view
+	        * no need to select anything.
+		*/
+	       temp_iter = iter;
+
+	       if (gtk_tree_model_iter_next (tree_model, &iter)) {
+	          path = gtk_tree_model_get_path (tree_model, &iter);
+	          row_reference = gtk_tree_row_reference_new (tree_model, path);
+	       } else {
+	          path = gtk_tree_model_get_path (tree_model, &temp_iter);
+	          if (gtk_tree_path_prev (path)) {
+	             row_reference = gtk_tree_row_reference_new (tree_model, path);
+	          }
 	       }
+	       gtk_tree_path_free (path);
 	   }
        
-	   gtk_tree_path_free (path);
+	   gtk_tree_path_free (file_path);
 		
 	   fm_list_model_remove_file (list_view->details->model, file);
-	   
-	   if(gtk_tree_row_reference_valid(row_reference)) {
-	      path = gtk_tree_row_reference_get_path (row_reference);
-	      gtk_tree_view_set_cursor (list_view->details->tree_view, path, NULL, FALSE);
-              gtk_tree_path_free (path);
-	      
+
+	   if (gtk_tree_row_reference_valid (row_reference)) {
+	      if (list_view->details->new_selection_path) {
+	          gtk_tree_path_free (list_view->details->new_selection_path);
+	      }
+	      list_view->details->new_selection_path = gtk_tree_row_reference_get_path (row_reference);
 	   }
 	  
 	   if(row_reference) {
@@ -1649,7 +1906,7 @@ action_visible_columns_callback (GtkAction *action,
 	}
 }
 
-static GtkActionEntry list_view_entries[] = {
+static const GtkActionEntry list_view_entries[] = {
   { "Visible Columns", NULL,                  /* name, stock id */
     N_("Visible _Columns..."), NULL,                /* label, accelerator */
     N_("Select the columns visible in this folder"),                   /* tooltip */ 
@@ -1879,7 +2136,9 @@ fm_list_view_can_zoom_out (FMDirectoryView *view)
 }
 
 static void
-fm_list_view_start_renaming_file (FMDirectoryView *view, NautilusFile *file)
+fm_list_view_start_renaming_file (FMDirectoryView *view,
+				  NautilusFile *file,
+				  gboolean select_all)
 {
 	FMListView *list_view;
 	GtkTreeIter iter;
@@ -1899,6 +2158,9 @@ fm_list_view_start_renaming_file (FMDirectoryView *view, NautilusFile *file)
 		return;
 	}
 
+	/* Freeze updates to the view to prevent losing rename focus when the tree view updates */
+	fm_directory_view_freeze_updates (FM_DIRECTORY_VIEW (view));
+
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (list_view->details->model), &iter);
 
 	/*Make filename-cells editable.*/
@@ -1922,8 +2184,13 @@ fm_list_view_start_renaming_file (FMDirectoryView *view, NautilusFile *file)
 	g_free (list_view->details->original_name);
 
 	list_view->details->original_name = g_strdup (gtk_entry_get_text (entry));
-	eel_filename_get_rename_region (list_view->details->original_name,
-					&start_offset, &end_offset);
+	if (select_all) {
+		start_offset = 0;
+		end_offset = -1;
+	} else {
+		eel_filename_get_rename_region (list_view->details->original_name,
+						&start_offset, &end_offset);
+	}
 	gtk_editable_select_region (GTK_EDITABLE (entry), start_offset, end_offset);
 	
 	gtk_tree_path_free (path);
@@ -2073,6 +2340,9 @@ fm_list_view_finalize (GObject *object)
 	if (list_view->details->double_click_path[1]) {
 		gtk_tree_path_free (list_view->details->double_click_path[1]);
 	}
+	if (list_view->details->new_selection_path) {
+		gtk_tree_path_free (list_view->details->new_selection_path);
+	}
 
 	gtk_target_list_unref (list_view->details->source_target_list);
 	
@@ -2188,6 +2458,7 @@ fm_list_view_class_init (FMListViewClass *class)
 	fm_directory_view_class->file_changed = fm_list_view_file_changed;
 	fm_directory_view_class->get_background_widget = fm_list_view_get_background_widget;
 	fm_directory_view_class->get_selection = fm_list_view_get_selection;
+	fm_directory_view_class->get_selection_for_file_transfer = fm_list_view_get_selection_for_file_transfer;
 	fm_directory_view_class->get_item_count = fm_list_view_get_item_count;
 	fm_directory_view_class->is_empty = fm_list_view_is_empty;
 	fm_directory_view_class->remove_file = fm_list_view_remove_file;
@@ -2204,6 +2475,7 @@ fm_list_view_class_init (FMListViewClass *class)
 	fm_directory_view_class->get_zoom_level = fm_list_view_get_zoom_level;
 	fm_directory_view_class->zoom_to_level = fm_list_view_zoom_to_level;
         fm_directory_view_class->emblems_changed = fm_list_view_emblems_changed;
+	fm_directory_view_class->end_file_changes = fm_list_view_end_file_changes;
 
 	eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_CLICK_POLICY,
 				       &click_policy_auto_value);
@@ -2303,7 +2575,11 @@ fm_list_view_supports_uri (const char *uri,
 static NautilusViewInfo fm_list_view = {
 	FM_LIST_VIEW_ID,
 	N_("List"),
-	N_("_List"),
+	N_("View as List"),
+	N_("View as _List"),
+	N_("The list view encountered an error."),
+	N_("The list view encountered an error while starting up."),
+	N_("Display this location with the list view."),
 	fm_list_view_create,
 	fm_list_view_supports_uri
 };
@@ -2312,7 +2588,11 @@ void
 fm_list_view_register (void)
 {
 	fm_list_view.label = _(fm_list_view.label);
-	fm_list_view.label_with_mnemonic = _(fm_list_view.label_with_mnemonic);
-	
+	fm_list_view.view_as_label = _(fm_list_view.view_as_label);
+	fm_list_view.view_as_label_with_mnemonic = _(fm_list_view.view_as_label_with_mnemonic);
+	fm_list_view.error_label = _(fm_list_view.error_label);
+	fm_list_view.startup_error_label = _(fm_list_view.startup_error_label);
+	fm_list_view.display_location_label = _(fm_list_view.display_location_label);
+
 	nautilus_view_factory_register (&fm_list_view);
 }
